-- File-Revision: 16
-- cooldown stuff

local find = string.find
local tremove = table.remove;
local tinsert = table.insert;

RCOptions = {};

local defaults = {disableBuff = false, disableDebuff = false};

local numTimers = 0;
local unusedTimers = {};
local activeCooldowns = {};

--@alpha@
function cooldownTimer_PrintDebug()
  DEFAULT_CHAT_FRAME:AddMessage("CooldownTimer Debug stuff");
  DEFAULT_CHAT_FRAME:AddMessage("numTimers: "..numTimers);
  DEFAULT_CHAT_FRAME:AddMessage("unusedTimers: "..#unusedTimers);
  
  for i=1,numTimers do 
    DEFAULT_CHAT_FRAME:AddMessage("timer #"..i.." has parent "..((_G["CooldownTimers"..i]:GetParent():GetName()) or "nil"));
  end
end
--@end-alpha@

-- Returns a useable timerobject. Creates one if there are none free to be used
local function getCooldownTimer(cooldown)
  local timer = tremove(unusedTimers);
  if ( not timer ) then
    numTimers = numTimers + 1;
    timer = CreateFrame("FRAME","CooldownTimers"..numTimers,RangeCooldownFrame,"CooldownTimerTemplate");
  end
   
  return timer
end

-- frees the given timer to be used later by calls to getCooldownTimer
local function freeCooldownTimer(cooldown)
  local timer = cooldown.timer;
  timer:Hide();
  timer:SetParent(RangeCooldownFrame);
  
  tinsert(unusedTimers, timer);
  cooldown.timer = nil;
end

SLASH_RANGECOOLDOWN1, SLASH_RANGECOOLDOWN2 = '/rangecooldown', '/rc'
function SlashCmdList.RANGECOOLDOWN(msg, editbox)
  local command, rest = msg:match("^(%S*)%s*(.-)$");
  
  if command == "buff" then
    if rest == "show" then
      RCOptions.disableBuff = false;
      RCPrintMessage("Buff Cooldowns Enabled");
    else
      RCOptions.disableBuff = true;
      RCPrintMessage("Buff Cooldowns Disabled");
    end  
  elseif command == "debuff" then
    if rest == "show" then
      RCOptions.disableDebuff = false;
      RCPrintMessage("Debuff Cooldowns Enabled");
    else
      RCOptions.disableDebuff = true;
      RCPrintMessage("Debuff Cooldowns Disabled");
    end
  else
    RCPrintMessage("Syntax: /rangecooldown (buff|debuff) (show||hide)");
  end
end

function RCPrintMessage(msg)
  DEFAULT_CHAT_FRAME:AddMessage("|cFF00FF00RangeCooldown: |r"..msg, .2, .4, 1.0);
end

-- updates the timer text if there is some time left
function cooldownTimer_OnUpdate(self, elapsed)
  self.elapsed = self.elapsed + elapsed;
  if self.elapsed > .1 then
    cooldownTimer_SetTime(self);
    self.elapsed = 0;
    if self.time == 0 then
      local cooldown = self:GetParent();
      freeCooldownTimer(cooldown);
    end
  end
end

-- Formats and then sets the time on the given timer
function cooldownTimer_SetTime(timer)
  local timeleft = timer.time - GetTime();

  if timeleft < 0 then
    timer.time = 0;
  elseif timeleft < 1 then
    timer.text:SetFontObject("CooldownFontSmall");
    timer.text:SetText(("%.1f"):format(timeleft));
  elseif timeleft < 60 then
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%d"):format(timeleft));
  elseif timeleft < 3600 then
    timeleft = floor(timeleft/60);
    timer.text:SetFontObject("CooldownFontMedium");
    timer.text:SetText(("%dm"):format(timeleft));
  else
    timeleft = floor(timeleft/3600);
    timer.text:SetFontObject("CooldownFontLarge");
    timer.text:SetText(("%dh"):format(timeleft));
  end
end

-- funtion hooked to the set timer function for cooldowns.  gets called for items and most cooldowns except action buttons.
function cooldownTimer_SetTimer(self, start, duration, enable, charges, maxcharges)
  --local button = self:GetParent();
  local name = self:GetName(); 

  -- grab the timer object for the given button
  if (RCOptions.disableBuff and find(name,'Buff')) or (RCOptions.disableDebuff and find(name,"Debuff")) then
    if self.timer then
      freeCooldownTimer(self);
    end
    return;
  end
 
  if not self:GetParent():IsShown() then
    return
  end

  -- if we have a start time and the duration is over the base gcd of 1.5 then set the time
  if (start and start > 0 and duration > 1.5 and enable > 0) then
    if not self.timer then  -- we dont have a timer currently so get one
      self.timer = getCooldownTimer(self);

      if (find(name,"Buff") or find(name,"Debuff")) then
        self.timer:SetScale(.5);
      else
        self.timer:SetScale(1.0);
      end
      
      self.timer:SetParent(self);
      self.timer:ClearAllPoints();
      self.timer:SetAllPoints();
      
      -- default to 0
      self.timer.elapsed = 0;
    end
    
    -- set the values for the timer
    self.timer.time = duration + start;
    self.timer:Show();
    cooldownTimer_SetTime(self.timer);
  else
    if self.timer then -- we no longer need the timer so free it up for use later
      freeCooldownTimer(self);
    end
  end
end

-- Functions to keep track of active cooldowns on action buttons
local function cooldown_OnShow(self)
  activeCooldowns[self] = true;
end

local function cooldown_OnHide(self)
  activeCooldowns[self] = nil;
  if self.timer then
    freeCooldownTimer(self);
  end
end

function cooldownTimer_AddAction(button, action, cooldown)
  if not cooldown.timerAction then
    cooldown:HookScript('OnShow', cooldown_OnShow);
    cooldown:HookScript('OnHide', cooldown_OnHide);
  end
  cooldown.timerAction = action;
end

-- function to check all active for cooldowns which match our requirements
function cooldownTimer_CheckCooldowns()
  local i, cooldown;
  for cooldown in pairs(activeCooldowns) do
    local start, duration, enable, charges, maxcharges = GetActionCooldown(cooldown.timerAction);
    cooldownTimer_SetTimer(cooldown, start, duration, enable, charges, maxcharges);
  end
end

do 
  hooksecurefunc("SetActionUIButton",cooldownTimer_AddAction);
  hooksecurefunc("CooldownFrame_SetTimer",cooldownTimer_SetTimer);
end

function cooldownTimerFrame_OnEvent(self, event, arg1)
  
  if event == "ACTIONBAR_UPDATE_COOLDOWN" then
    cooldownTimer_CheckCooldowns();
    return;
  end
  
  if event == "PLAYER_LOGIN" then
    local i, button;
    
    -- Add all the current actions to the list
    for i, button in pairs(ActionBarButtonEventsFrame.frames) do
      cooldownTimer_AddAction(button, button.action, button.cooldown);
    end
    
    cooldownTimer_CheckCooldowns();
    
    self:UnregisterEvent("PLAYER_LOGIN");
    return
  end
  
  if event == "ADDON_LOADED" then
    if arg1 == "RangeCooldown" then
      -- load our defaults if no settings are found
      local key, value;
      for key, value in pairs(defaults) do
        if RCOptions[key] == nil then
          RCOptions[key] = value;
        end
      end      
      self:UnregisterEvent("ADDON_LOADED");
    end
    return;
  end

end

